;;; checkdoc-backslash.el --- check \ backslashing in docstrings

;; Copyright 2014, 2017 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 1
;; Keywords: lisp, maint
;; URL: http://user42.tuxfamily.org/checkdoc-backslash/index.html

;; checkdoc-backslash.el is free software; you can redistribute it
;; and/or modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 3, or (at your
;; option) any later version.
;;
;; checkdoc-backslash.el is distributed in the hope that it will be
;; useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.

;;; Commentary:

;; This spot of code checks that docstring backslash \ forms are confined to
;; the documented \[ etc.  See the `checkdoc-backslash-docstring' docstring
;; below for more details.

;;; Emacsen:

;; Designed for Emacs 21 and up, works in XEmacs 21 too.

;;; Install:

;; Put checkdoc-backslash.el in one of your `load-path' directories, and
;; in your .emacs add
;;
;;     (autoload 'checkdoc-backslash-docstring "checkdoc-backslash")
;;     (add-hook 'checkdoc-style-hooks 'checkdoc-backslash-docstring)
;;
;; There's an autoload cookie for the function if you install by
;; `M-x package-install' or know how to use `update-file-autoloads', then
;; add or customize `checkdoc-style-hooks'.

;;; History:

;; Version 1 - the first version

;;; Code:

(require 'checkdoc)

;;;###autoload
(defun checkdoc-backslash-docstring (defuninfo endpoint)
  "Check backslash forms in docstrings.
This function is designed for use in `checkdoc-style-hooks'.

Info node \"(elisp)Keys in Documentation\" describes the possible
backslash sequences in docstrings

    \\=\\[   command key sequence
    \\=\\{   keymap variable
    \\=\\<   keymap variable for key sequence
    \\=\\=   literal next character, such as \\=\\=\\ literal backslash

This check asks that the only backslashing used in docstrings is
these forms.  The fix offered for other backslashes is to escape
as \\=\\=\\=\\ (which will appear as \\=\\\\=\\=\\=\\\\=\\ to escape as a Lisp string).

This escaping guards against some new backslash form applying to
your docstring.  Perhaps new backslash forms are unlikely, or
would be only be on with some unusual character anyway, in which
case the escaping here would be unnecessary.

----
The checkdoc-backslash.el home page is
URL `http://user42.tuxfamily.org/checkdoc-backslash/index.html'"

  (let (ret)
    (while (and (not ret)
                (re-search-forward "\
\\\\\
\\\\\
\\(\
\\([^[{<=]\\)\
\\|\
=\\(\
\\\\\
\\\\\
\\|\
\[^\\\\]\
\\)\\)" endpoint t))
      (when (match-beginning 2)
        (if (checkdoc-autofix-ask-replace
             (copy-marker (match-beginning 0))
             (copy-marker (match-end 0))
             "Escape backslash with \\\\=\\\\? "
             "\\=\\"
             t) ;; complex, meaning ask the user
            ;; fix applied, keep looping
            nil
          ;; not applied, stop like builtin checks do
          (setq ret (checkdoc-create-error
                     "Backslashes in docstrings should be escaped as \\=\\"
                     (match-beginning 0)
                     (match-end 0))))))
    ret))

;;;###autoload
(custom-add-option 'checkdoc-style-hooks 'checkdoc-backslash-docstring)

(provide 'checkdoc-backslash)

;;  LocalWords:  docstring docstrings

;;; checkdoc-backslash.el ends here
